"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = exports.logout = exports.checkToken = exports.login = void 0;
const getUser_1 = require("../utils/getUser");
const bcrypt = require("bcryptjs");
const jwt = require("jsonwebtoken");
const dataWithoutPassword_1 = require("../utils/dataWithoutPassword");
const student_record_1 = require("../records/student.record");
const checkMailAvailable_1 = require("../utils/checkMailAvailable");
const errors_1 = require("../utils/errors");
const login = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, password } = req.body;
        const user = yield (0, getUser_1.getUserWithRoleByEmail)(email);
        if (!user) {
            return res.status(401).json({ message: 'Invalid credentials' });
        }
        if (user) {
            const passwordMatch = yield bcrypt.compare(password, user.password);
            if (passwordMatch) {
                const token = jwt.sign({ userId: user.id }, 'easy-secret-key', { expiresIn: '1h' });
                res.json({
                    user: (0, dataWithoutPassword_1.userWithoutPassword)(user),
                    token,
                });
            }
            else {
                return res.status(401).json({ message: 'Invalid credentials' });
            }
        }
    }
    catch (err) {
        next(err);
    }
});
exports.login = login;
const checkToken = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const token = req.header('Authorization').slice(7);
    if (token) {
        try {
            const decoded = jwt.verify(token, 'easy-secret-key');
            if (typeof decoded === 'object' && 'userId' in decoded) {
                const userId = decoded.userId; // Ręczne rzutowanie do JwtPayload
                const user = yield (0, getUser_1.getUserById)(userId);
                if (user) {
                    const clearedUser = (0, dataWithoutPassword_1.userWithoutPassword)(user);
                    return res.status(200).json(clearedUser);
                }
            }
            else {
                console.error('Invalid token format');
            }
        }
        catch (err) {
            console.error('Token verification failed:', err);
        }
    }
    return res.status(401).json({ error: 'Unauthorized' });
});
exports.checkToken = checkToken;
const logout = (req, res, next) => {
    res.status(200).json({ message: 'Logged out successfully' });
};
exports.logout = logout;
const register = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { email, password } = req.body;
    const hashedPassword = yield bcrypt.hash(password, 10);
    const checkOkMail = yield (0, checkMailAvailable_1.checkMailAvailable)(email); //sprawdzanie dostępności maila
    if (!checkOkMail) {
        throw new errors_1.ValidationError("Email already exists.");
    }
    const userData = Object.assign(Object.assign({}, req.body), { name: null, last_name: null, password: hashedPassword });
    const user = yield new student_record_1.StudentRecord(userData);
    yield user.insert();
    res.json({
        student: (0, dataWithoutPassword_1.userWithoutPassword)(user)
    });
});
exports.register = register;
//# sourceMappingURL=auth.js.map